/*
	
	FontDisplaySetting.cpp
	
	Copyright 1998 HAMANAKA Makoto , All Rights Reserved.
	
*/

#ifndef FONT_DISPLAY_SETTING_H
#include "FontDisplaySetting.h"
#endif
#ifndef FONT_VIEW_H
#include "FontView.h" // only for FONT_DEFAULT_FONT_SIZE
#endif

#include <stdio.h>

#ifndef RGB_COLOR_UTIL_H
#include "rgb_color_util.h"
#endif


FontDisplaySetting::FontDisplaySetting()
	: setting_name("*no name*")
	, enable_display_lines(true)
	, enable_antialiasing(true)
	, black_white_mode(true)
	, font_size(FONT_DEFAULT_FONT_SIZE)
	, fg_color(make_rgb_color(0,0,0))
	, bg_color(make_rgb_color(0xff,0xff,0xff))
{
/*	rgb_color _fg_color = {0,0,0,0xff};
	rgb_color _bg_color = {0xff,0xff,0xff,0xff};
	this->fg_color = _fg_color;
	this->bg_color = _bg_color;
*/
}

FontDisplaySetting::FontDisplaySetting(FontDisplaySetting &copy_src)
	: setting_name(copy_src.setting_name)
	, enable_display_lines(copy_src.enable_display_lines)
	, enable_antialiasing(copy_src.enable_antialiasing)
	, black_white_mode(copy_src.black_white_mode)
	, font_size(copy_src.font_size)
	, fg_color(copy_src.fg_color)
	, bg_color(copy_src.bg_color)
{
}

void FontDisplaySetting::SaveToMessage(BMessage *target)
{
	target->AddString("setting_name",setting_name);
	target->AddBool("enable_display_lines",enable_display_lines);
	target->AddBool("enable_antialiasing",enable_antialiasing);
//	target->SetBool("black_white_mode",black_white_mode);
	target->AddFloat("font_size",font_size);
	target->AddInt32("fg_color",(int32)make_uint32_from_rgb_color(fg_color));
	target->AddInt32("bg_color",(int32)make_uint32_from_rgb_color(bg_color));
}

void FontDisplaySetting::LoadFromMessage(BMessage *target)
{
	status_t result;
	
	// *****  load setting name  *****
	
	BString _setting_name;
	result = target->FindString("setting_name", &_setting_name);
	if (result == B_OK)
		this->setting_name = _setting_name;
	else
	{
		printf("FontDisplaySetting::LoadFromMessage load setting error : setting_name"
			" :result = %lux", result);
		this->setting_name = "read_error"; // set default
	}
	
	// *****  load enable_display_lines  *****
	
	bool _enable_display_lines;
	result = target->FindBool("enable_display_lines", &_enable_display_lines);
	if (result == B_OK)
		this->enable_display_lines = _enable_display_lines;
	else
	{
		printf("FontDisplaySetting::LoadFromMessage load setting error : enable_display_lines"
			" :result = %lux", result);
		this->enable_display_lines = true; // set default
	}
	
	// *****  load enable_antialiasing  *****
	
	bool _enable_antialiasing;
	result = target->FindBool("enable_antialiasing", &_enable_antialiasing);
	if (result == B_OK)
		this->enable_antialiasing = _enable_antialiasing;
	else
	{
		printf("FontDisplaySetting::LoadFromMessage load setting error : enable_antialiasing"
			" :result = %lux", result);
		this->enable_antialiasing = true; // set default
	}
	// *****  load black_white_mode  *****
	/*
	bool _black_white_mode;
	result = target->FindBool("black_white_mode",&_black_white_mode);
	if (result == B_OK)
		this->black_white_mode = _black_white_mode;
	else
	{
		printf("FontWorldSetting::LoadFromMessage load setting error : black_white_mode"
			" :result = %lux", result);
		this->black_white_mode = true; // set default
	}
	*/
	
	// *****  load font_size  *****
	
	float _font_size;
	result = target->FindFloat("font_size", &_font_size);
	if (result == B_OK)
		this->font_size = _font_size;
	else
	{
		printf("FontDisplaySetting::LoadFromMessage load setting error : font_size"
			" :result = %lux", result);
		this->font_size = 48.0; // set default
	}
	
	// *****  load fg_color  *****
	
	int32 _fg_color;
	result = target->FindInt32("fg_color", &_fg_color);
	if (result == B_OK)
		this->fg_color = make_rgb_color_from_uint32((uint32)_fg_color);
	else
	{
		printf("FontDisplaySetting::LoadFromMessage load setting error : fg_color"
			" :result = %lux", result);
		this->fg_color = make_rgb_color(0,0,0); // set default
	}
	
	// *****  load bg_color  *****
	
	int32 _bg_color;
	result = target->FindInt32("bg_color", &_bg_color);
	if (result == B_OK)
		this->bg_color = make_rgb_color_from_uint32((uint32)_bg_color);
	else
	{
		printf("FontDisplaySetting::LoadFromMessage load setting error : bg_color"
			" :result = %lux", result);
		this->bg_color = make_rgb_color(0xff,0xff,0xff); // set default
	}

}

BString FontDisplaySetting::ToString()
{
	BString string;

	string << "< FontDisplaySetting >\n";
	string << "  | Name :" << setting_name << "\n";
	string << "  | enable_display_lines :" << (enable_display_lines?"true":"false") << "\n";
	string << "  | enable_antialiasing :" << (enable_antialiasing?"true":"false") << "\n";
	string << "  | black_white_mode :" << (black_white_mode?"true":"false") << "\n";
	string << "  | font_size :" << font_size << "\n";
	string << "  | fg_color :" << rgb_color_to_string(fg_color) << "\n";
	string << "  | bg_color :" << rgb_color_to_string(bg_color) << "\n";
	
	return string;
}

/*
bool FontDisplaySetting::QuitRequested()
{
//	be_app->PostMessage(B_QUIT_REQUESTED);
//	return(true);
}
*/
