/*
	
	FontView.cpp
	
	Copyright 1998 HAMANAKA Makoto, All Rights Reserved.
	
*/

#ifndef FONT_VIEW_H
#include "FontView.h"
#endif

#include <stdio.h>
#include <BitmapStream.h>
#include <Directory.h>
#include <Entry.h>
#include <NodeInfo.h>
#include <TranslationUtils.h> 
#ifndef _HANDLER_H
#include <Handler.h>
#endif

FontView::FontView(BRect rect, const char *name, const char *label,
					BMessage *message, pbfont_stl_vector &fonts)
	   	   : BControl(rect, name, label, message, B_FOLLOW_ALL, B_WILL_DRAW | B_FRAME_EVENTS )
	   	   , fonts(fonts)
	   	   , scroller(NULL)
	   	   , text(NULL)
	   	   , font_item_height(FONT_ITEM_HEIGHT)
	   	   , fontsize(FONT_ITEM_HEIGHT)
	   	   , font_drawing_mode(FONTVIEW_SHOW_LINES_FLAG)
	   	   , is_antialiasing_disabled(false)
	   	   , is_required_to_adjust_scroll_ranges(false)
	   	   , selected_item_index(-1)
	   	   , popupmenu(NULL)
	   	   , fontitem_offset_left(FONTITEM_OFFSET_LEFT_DEFAULT)
	   	   , drag_clip_suffix("")
	   	   , drag_clip_mime_type("")
/*	   	   , all_translators(0)
	   	   , all_translators_num(0)
	   	   , all_translators_output_type_strings(0)
	   	   , all_translators_output_types_num(0)
*/
			,background_bitmap(0)
			, dnd_output_format("auto")
{
//	this->fonts = fonts;
	this->fonts_num = fonts.size();	
	this->SetText("text not specified.");
	this->SetTextFontSize(FONT_DEFAULT_FONT_SIZE);
//	this->InitTranslators();

	background_bitmap = BTranslationUtils::GetBitmapFile("image");

	
//	SetFont(be_bold_font);
//	SetFontSize(24);
	fgcolor.red = 0;
	fgcolor.green = 0;
	fgcolor.blue = 0;
	fgcolor.alpha = 255;
	bgcolor.red = 255;
	bgcolor.green = 255;
	bgcolor.blue = 255;
	bgcolor.alpha = 255;
}

FontView::~FontView()
{
	if (text != NULL) delete [] text;
	text = NULL;

	if (popupmenu != NULL) delete popupmenu;
	popupmenu = NULL;
}
/*
FontDisplaySetting::
	bool enable_display_lines;
	bool enable_antialiasing;
	bool black_white_mode;
	float font_size;
	rgb_color fg_color;
	rgb_color bg_color;
*/

void FontView::ApplySetting(FontDisplaySetting *_setting)
{
//	this->font_drawing_mode = (this->font_drawing_mode & ~FONTVIEW_SHOW_LINES_FLAG)
//						| (_setting->enable_display_lines) ? FONTVIEW_SHOW_LINES_FLAG : 0;
//	this->is_antialiasing_disabled = !_setting->enable_antialiasing;
//	this->fontsize = (int32)_setting->font_size;

	this->SetFontDrawingMode((this->font_drawing_mode & ~FONTVIEW_SHOW_LINES_FLAG)
						| (_setting->enable_display_lines) ? FONTVIEW_SHOW_LINES_FLAG : 0);
	this->DisableAntialiasing(!_setting->enable_antialiasing);
	this->SetTextFontSize((int32)_setting->font_size);

	this->SetFGColor(_setting->fg_color);
	this->SetBGColor(_setting->bg_color);
//	this->fgcolor = _setting->fg_color;
//	this->bgcolor = _setting->bg_color;
	
}

void FontView::StoreSetting(FontDisplaySetting *_setting)
{
	_setting->enable_display_lines = ((this->font_drawing_mode & FONTVIEW_SHOW_LINES_FLAG) == FONTVIEW_SHOW_LINES_FLAG);
	_setting->enable_antialiasing = !this->is_antialiasing_disabled;
	_setting->font_size = this->fontsize;
	_setting->fg_color = this->fgcolor;
	_setting->bg_color = this->bgcolor;
}


void FontView::SetPopUpMenu(BPopUpMenu *new_popupmenu)
{
	if (popupmenu != NULL) delete popupmenu;
	popupmenu = NULL;

	popupmenu = new_popupmenu;
}

void FontView::AttachedToWindow(void) 
{
	if (this->LockLooper())
	{
		this->SetViewColor(bgcolor);
		this->UnlockLooper();
	}

	if(background_bitmap != NULL)
	{
		if (this->LockLooper())
		{
			this->SetViewBitmap(background_bitmap, B_FOLLOW_NONE);
			delete background_bitmap;
			this->UnlockLooper();
		}
	}

	this->AdjustScrollRanges();
}

/*
SetBackgroundBitmap()
{
	background_bitmap = BTranslationUtils::GetBitmapFile("image");
}
*/
void FontView::DisableAntialiasing(bool new_mode)
{
	if (is_antialiasing_disabled != new_mode)
	{
		for (int i=0 ; i<fonts_num ; i++)
		{
			if (new_mode)
			{
				fonts[i]->SetFlags(fonts[i]->Flags() | B_DISABLE_ANTIALIASING);
			}
			else
			{
				fonts[i]->SetFlags(fonts[i]->Flags() & ~B_DISABLE_ANTIALIASING);
			}
		}
	}

	is_antialiasing_disabled = new_mode;

	if (Window()->Lock())
	{
		Invalidate();
		Window()->Unlock();
	}
}

bool FontView::IsAntialiasingEnabled()
{
	return is_antialiasing_disabled;
}

void FontView::SetFontDrawingMode(int32 new_mode)
{
	font_drawing_mode = new_mode;

	if (Window()->Lock())
	{
		Invalidate();
		Window()->Unlock();
	}
}

int32 FontView::FontDrawingMode(void)
{
	return font_drawing_mode;
}

void FontView::SetText(const char *new_text)
{
	if (text != NULL) delete [] text;
	text = NULL;
	
	int length = strlen(new_text);
	text = new char[length+1];
	if (text == NULL) return;
	
	strncpy(text, new_text, length);
	text[length] = '\0';
	if (Window()->Lock())
	{
		Invalidate();
		Window()->Unlock();
	}
}

void FontView::SetFGColor(rgb_color _color)
{
	this->fgcolor = _color;
	if (Window()->Lock())
	{
		Invalidate();
		Window()->Unlock();
	}	
}

void FontView::SetBGColor(rgb_color _color)
{
	this->bgcolor = _color;
	if (Window()->Lock())
	{
		SetViewColor(this->bgcolor);
		Invalidate();
		Window()->Unlock();
	}	
}

char *FontView::Text()
{
	return text;
}

void FontView::SetTextFontSize(int32 new_font_size)
{
	float old_font_item_height = font_item_height;
	font_item_height = (int)((double)new_font_size * 1.5) + 2;
	fontsize = new_font_size;

	if (scroller == NULL)
	{
		return;
	}
	BScrollBar *scrollbar = scroller->ScrollBar(B_VERTICAL);
	if (scrollbar == NULL)
	{
		return;
	}

	if (Window()->Lock())
	{
		Invalidate();
		//Window()->BeginViewTransaction();
		float min,max;
		scrollbar->GetRange(&min,&max);
		float degree = (scrollbar->Value() - min)/ (float)old_font_item_height;		
		//---------------------

		this->AdjustScrollRanges();

		//---------------------
		scrollbar->GetRange(&min,&max);

		float new_value;
		new_value = min + ((float)font_item_height * degree);
		scrollbar->SetValue(new_value);
		
		//Window()->EndViewTransaction();
		Window()->Unlock();
	}
}

void FontView::SetFonts(pbfont_stl_vector &fonts)
{
	if (Window()->Lock())
	{
		this->fonts = fonts;
		this->fonts_num = fonts.size();
		this->Invalidate();
		Window()->Unlock();
	}
}

void FontView::AdjustScrollRanges()
{
	if (Window()->Lock())
	{
		if (scroller != NULL)
		{
			int bigstep;
			if (Bounds().Height() > font_item_height * 2)
			{
				bigstep = ((int)Bounds().Height() / font_item_height - 1) * font_item_height;
			}
			else if (Bounds().Height() > font_item_height)
			{
				bigstep = font_item_height;
			}
			else
			{
				bigstep = (int)(Bounds().Height() * 0.8);
			}
			BScrollBar *scrollbar = NULL;
			scrollbar = scroller->ScrollBar(B_VERTICAL);
			if (scrollbar != NULL)
			{
				scrollbar->SetSteps(font_item_height, bigstep);
				int range_max = font_item_height * fonts_num - bigstep;
				if (range_max < 1) range_max = 1;
				scrollbar->SetRange(1, range_max);
				//Invalidate();
			}
		}
		Window()->Unlock();
	}
	is_required_to_adjust_scroll_ranges = false;
}


void FontView::FrameResized(float width, float height)
{
	// imi nai rashii
//	printf("resized");
	AdjustScrollRanges();
}


void FontView::FrameMoved(BPoint parentPoint)
{
/*	if (is_required_to_adjust_scroll_ranges == true)
	{
		this->AdjustScrollRanges();
	}
*/}

void FontView::TargetedByScrollView(BScrollView *scroller1)
{
//	printf("targetted\n");
	// ue no imi ga nai node iranai darou
	scroller = scroller1;
}

/*
void FontView::SetMessageOnMouseDown(BMessage *new_message,BLooper *target)
{
	if (message != NULL) delete message;
	message = NULL;
	if (target != NULL) delete target;
	target = NULL;

	message = new_message;
	target = new_target;
}
*/

void FontView::MouseDown(BPoint point)
{
	uint32 buttons;
	BPoint mouse_point;
	this->GetMouse(&mouse_point, &buttons);
	if (buttons & B_PRIMARY_MOUSE_BUTTON)
	{
		MouseDownToDrag(point);
		return;
	}

		int32 index = this->GetFontIndexOnLocation(point);
		selected_item_index = index;
		if (index < 0) return;
		
		DrawAnFontItem(index);
		
		BMenuItem *selected; 
//		BMessage *copy; 
		ConvertToScreen(&point); 
		selected = popupmenu->Go(point); 
		
		if ( selected )
		{ 
			BLooper *looper; 
			BHandler *target = selected->Target(&looper); 
			BMessage *message = selected->Message();
			message->RemoveName("font_index");
			message->AddInt32("font_index", index);
			// post message to show 'font info'
			looper->PostMessage(message, target); 
		}
	
		selected_item_index = -1;
		if (index >= 0 && index < fonts_num)
		{
			//DrawAnFontItem(index);
			BRect rect = this->GetBoundsByIndex(index);
			this->Invalidate(rect);
		}
}

int32 FontView::GetFontIndexOnLocation(BPoint point)
{
	int index = (int)point.y / font_item_height;
	if (index >= fonts_num) return -1;
	return index;
}

BFont *FontView::GetFontByIndex(int32 index)
{
	if (index < 0 && index >= fonts_num) return NULL;
	
	return (fonts[index]);
}

BRect FontView::GetBoundsByIndex(int32 index)
{
		BRect rect = this->Bounds();
		rect.top = index*font_item_height + 0.5;
		rect.bottom = index*font_item_height + font_item_height -0.5;
		return rect;
}

void FontView::Draw(BRect updateRect)
{
	if (font_item_height <= 0 || font_item_height > 1000) return;

	int start_num = ((int) updateRect.top / font_item_height) -1;
	if (start_num <0) start_num = 0;
	int end_num = ((int)updateRect.bottom / font_item_height) + 1;

	//this->SetHighColor( rand()*255/RAND_MAX ,rand()*255/RAND_MAX ,rand()*255/RAND_MAX );
	
	//printf("updateRect.top:%6.3f bottom:%6.3f start:%d end:%d \n"
	//	,updateRect.top ,updateRect.bottom, start_num, end_num);
	
	for (int i=start_num ; i<end_num && i<fonts_num; i++)
	{
		this->DrawAnFontItem(i);
	}
}

void FontView::DrawAnFontItem(int index)
{
		if (index < 0 || index >= fonts_num) return;
		BFont *font = fonts[index];
		if (font == NULL) return;

		// rectangle of this item (on the screen)
		BRect rect = this->Bounds();
		rect.top = index * font_item_height + 0.5;
		rect.bottom = index * font_item_height + font_item_height -0.5;

		// background
//		this->SetHighColor(bgcolor);
//		this->FillRect(rect);
		
		
		// set font color
		SetHighColor(fgcolor);
		SetLowColor(bgcolor);
		SetViewColor(bgcolor);
		SetDrawingMode(B_OP_COPY);

		// family & style
		font_family family;
		font_style style;
		font->GetFamilyAndStyle(&family, &style);
		SetFont(be_plain_font);
		DrawString(family, BPoint(10 , index * font_item_height + font_item_height - 5));
		DrawString(style, BPoint(120 , index * font_item_height + font_item_height - 5));

		// set size to font
		font->SetSize(fontsize);

		// get font height
		font_height fontheight;
		font->GetHeight(&fontheight);

		double base_y = index * font_item_height + font_item_height;
		double baseline_y = base_y -1.0 - fontheight.descent;
		double descent_y = base_y - 1.0;
		double ascent_y = base_y - 1.0 - fontheight.descent - fontheight.ascent;


		if (font_drawing_mode & FONTVIEW_SHOW_LINES_FLAG)
		{
			// baseline
			SetHighColor(255,200,200);
			BPoint start(this->fontitem_offset_left, baseline_y);
			BPoint end(this->Bounds().right, baseline_y);
			this->StrokeLine(start, end);
			
			// descent
			SetHighColor(200,200,250);
			BPoint start2(this->fontitem_offset_left, descent_y);
			BPoint end2(this->Bounds().right, descent_y);
			this->StrokeLine(start2, end2);
			
			// ascent
			SetHighColor(200,250,200);
			BPoint start3(this->fontitem_offset_left, ascent_y);
			BPoint end3(this->Bounds().right, ascent_y);
			this->StrokeLine(start3, end3);
		}
		
		// main text
		SetHighColor(fgcolor);
		SetFont(font);
		DrawString(text, BPoint(this->fontitem_offset_left , baseline_y));

		// selected box
		if (index == selected_item_index)
		{
			this->SetHighColor(0,0,255);
			this->StrokeRect(rect);
		}

}

bool FontView::MouseDownToDrag(BPoint where)
{
	int32 index = GetFontIndexOnLocation(where);
	if (index < 0 || index >= fonts_num) return false;
	BFont *font = fonts[index];
	if (font == NULL) return false;

//	printf("index = %ld\n",index);
	
	// get edges ( not to print strings out of the rect )
	float left_edge = 1.0;
	float right_edge = 1.0;
	int len = strlen(text);
	if (len > 0)
	{
		edge_info *edges = new edge_info[len];
		if (edges != 0)
		{
			font->GetEdges(text, len, edges);
			if (edges[0].left < 0) left_edge = -edges[0].left * (double)fontsize + 1.0;
			if (edges[len-1].right < 0) right_edge = -edges[len-1].right * (double)fontsize + 1.0;
			printf("right_edge = %f\n", right_edge);
			printf("left_edge = %f\n", left_edge);
			delete [] edges;
		}
	}
	BPoint edge_offset(left_edge, 0);


	// get heights of the font
	font_height fontheight;
	font->GetHeight(&fontheight);

	BRect r = this->GetBoundsByIndex(index);
	BRect r_dragged = r;
	r_dragged.left = 0;
	/* REMIND : add fontsize/2 to right boundary */
	/* because font 'Embassy' and 'Comercial-Script' Go Over Right edge. */
	r_dragged.right = font->StringWidth(text) + ((double)fontsize/2.0) + right_edge;
	r_dragged.top = r_dragged.bottom - fontheight.ascent - fontheight.descent;

	BRect r_bmp = r_dragged;
	r_bmp.OffsetTo(0,0);
	r_bmp.right += left_edge;

	BRect r_bmp_dragging = r_bmp;
	if (r_bmp_dragging.right > 300.0) r_bmp_dragging.right = 300.0;
	if (r_bmp_dragging.bottom > 100.0) r_bmp_dragging.bottom = 100.0;


//	int zz = (int)(Bounds().right-r.right)/2;
//	r.OffsetBy(zz, zz);
	if (r.Contains(where)) {
		//	wait for a drag to start
		while (true) {
			BPoint w2;
			uint32 mods;
			GetMouse(&w2, &mods);
			if (!mods) {
				//	releasing the buttons without moving means no drag
				return false;
			}
			if (w2 != where) {
				//	moving the mouse starts a drag
				break;
			}
			snooze(40000);
		}
		BMessage msg(B_SIMPLE_DATA);
		//	add the types
		msg.AddString("be:types", B_FILE_MIME_TYPE);

		MimetypeTranslatorMap::iterator it;

		if (dnd_output_format != "auto" && dnd_output_format != ""
			&& (it = myApp->translators_map.lower_bound(dnd_output_format)) != myApp->translators_map.end())
		{// format is specified 
			msg.AddString("be:types", (it->first).c_str());
			msg.AddString("be:filetypes", (it->first).c_str());	
		}
		else
		{// 'auto' mode : accept every format that can translate
			MimetypeTranslatorMap::reverse_iterator rit;
			for (rit = myApp->translators_map.rbegin() ; rit != myApp->translators_map.rend() ; rit++)
			{
				msg.AddString("be:types", (rit->first).c_str());
				msg.AddString("be:filetypes", (rit->first).c_str());
			}
		}
		//	add the actions
		msg.AddInt32("be:actions", B_COPY_TARGET);
//		msg.AddInt32("be:actions", B_TRASH_TARGET);

		string clip_name("FontWorld_image_clip");
		clip_name += drag_clip_suffix;

		msg.AddString("be:clip_name", clip_name.c_str());

/* test of drop on ShowImage
		BEntry entry("/boot/home/Desktop/FontWorld_image_clip 2");
		entry_ref ref;
		entry.GetRef(&ref);
		msg.AddRef("refs",&ref);
*/
		msg.AddRect("fontworld:rect", r_bmp);
		msg.AddInt32("fontworld:font_index", index);
		msg.AddPoint("fontworld:edge_offset", edge_offset);
		
		rgb_color high = fgcolor;
		high.alpha = 230;
		rgb_color low;
		low.red = 0;
		low.green = 0;
		low.blue = 0;
		low.alpha = 0;
		BBitmap *bmp = this->CreateStringBitmap(font, r_bmp_dragging, text, high, 
												low, edge_offset, true);
		if (bmp != 0)
		{ // draw edge
			BView drawview(r_bmp_dragging, "draw_to_bmp", B_FOLLOW_NONE, B_WILL_DRAW);
			if (bmp->Lock())
			{
				bmp->AddChild(&drawview);
				
				BRect r_bmp_frame = r_bmp_dragging;
				r_bmp_frame.right -= 0.5;
				r_bmp_frame.bottom -= 0.5;

				drawview.SetDrawingMode(B_OP_ALPHA);
				drawview.SetHighColor(128,128,128,128);
				// bottom line
				drawview.StrokeLine(r_bmp_frame.LeftBottom(), r_bmp_frame.RightBottom());
				// left line
				drawview.StrokeLine(r_bmp_frame.LeftTop(), r_bmp_frame.LeftBottom());
				// right line
				if (r_bmp_dragging.right < r_bmp.right)
					drawview.SetHighColor(255,0,0,200);
				else
					drawview.SetHighColor(128,128,128,128);
				drawview.StrokeLine(r_bmp_frame.RightTop(), r_bmp_frame.RightBottom());
				// top line
				if (r_bmp_dragging.bottom < r_bmp.bottom)
					drawview.SetHighColor(255,0,0,200);
				else
					drawview.SetHighColor(128,128,128,128);
				drawview.StrokeLine(r_bmp_frame.LeftTop(), r_bmp_frame.RightTop());
				
				bmp->RemoveChild(&drawview);
				bmp->Unlock();
			}
		}

		// drag it!
		BPoint pt(where.x - r_dragged.left - this->fontitem_offset_left + left_edge, where.y - r_dragged.top);
		if (pt.x > r_bmp_dragging.right) pt.x = r_bmp_dragging.right;
		if (pt.y > r_bmp_dragging.bottom) pt.y = r_bmp_dragging.bottom;
		if (pt.y < 0) pt.y = 0;
		if (pt.x < 0) pt.x = 0;
		
		DragMessage(&msg, bmp, B_OP_ALPHA, pt, this);
		return true;
	}
	return false;
}

void FontView::MessageReceived(BMessage * request)
{
printf("FontView::MessageReceived\n");

	switch (request->what)
	{
	case B_COPY_TARGET:
		ActionCopy(request);
		break;
	}
}

void FontView::ActionCopy(BMessage * request)
{
printf("FontView::actioncopy\n");

	const char * type = NULL;

	const BMessage *previous;
	BFont *font;
	int32 index;
	BRect rect;
	BPoint edge_offset;
	if ((previous = request->Previous()) == NULL) return;
	if (previous->FindInt32("fontworld:font_index", &index) != B_NO_ERROR)
		return;
	if (index < 0 || index >= fonts_num)
		return;
	font = fonts[index];
	if (font == 0)
		return;
	if (previous->FindRect("fontworld:rect", &rect) != B_NO_ERROR)
		return;
	if (previous->FindPoint("fontworld:edge_offset", &edge_offset) != B_NO_ERROR)
		return;
	if (request->FindString("be:types", &type) != B_NO_ERROR)
		return;
		
	rgb_color low_color = this->bgcolor;
	low_color.alpha = 0;
	BBitmap *bmp = this->CreateStringBitmap(font, rect, text, fgcolor, low_color,
											edge_offset, false);
	BBitmapStream strm(bmp);

	if (!strcasecmp(type, B_FILE_MIME_TYPE))
	{
		printf("reply by file \n");
		const char * name;
		entry_ref dir;
		if (!request->FindString("be:filetypes", &type) &&
				!request->FindString("name", &name) &&
				!request->FindRef("directory", &dir) )
		{
			// find trasnlator from map
			MimetypeTranslatorMap::iterator it;
			string type_string(type);
			printf("type : %s\n",type);

			if ((it = myApp->translators_map.lower_bound(type_string)) != myApp->translators_map.end())
			{
				printf("write to this file!\n");
				translator_id translator = (it->second).first;
				int32 translator_code = (it->second).second;
			printf("translator_id = %ld\n", translator);
			printf("translator_code = %ld\n", translator_code);

				//	write file
				BDirectory d(&dir);
				BFile f(&d, name, O_RDWR | O_TRUNC);
				status_t result;
				if ((result = BTranslatorRoster::Default()->Translate(translator,
					&strm, NULL, &f, translator_code))!= B_NO_ERROR)
				{
					printf("translation error : code = %lx\n",result);
				}
				BNodeInfo ni(&f);
				ni.SetType(type);
			}
		}
	}
	else
	{
		printf("reply by message \n");
		// find trasnlator from map

		MimetypeTranslatorMap::iterator it;
		string type_string(type);
		printf("type : %s\n",type);
		if ((it = myApp->translators_map.lower_bound(type_string)) != myApp->translators_map.end())
		{
				translator_id translator = (it->second).first;
				int32 translator_code = (it->second).second;
			printf("translator_id = %ld\n", translator);
			printf("translator_code = %ld\n", translator_code);

			//	put in message
			BMessage msg(B_MIME_DATA);
			BMallocIO f;
			status_t result;
			if ((result = BTranslatorRoster::Default()->Translate(translator,
					&strm, NULL, &f, translator_code))!= B_NO_ERROR)
			{
				printf("translation error : code = %lx\n",result);
			}
			msg.AddData(type, B_MIME_TYPE, f.Buffer(), f.BufferLength());
			request->SendReply(&msg);
		}
	}
	strm.DetachBitmap(&bmp);
	delete bmp;
}

BBitmap *FontView::CreateStringBitmap(BFont *font, BRect rect, char *text, rgb_color high_color, rgb_color low_color, BPoint edge_offset, bool transparent_mode)
{
	BBitmap *bmp = new BBitmap(rect, B_RGB32, true/* accept views */);
	if (bmp == 0) return 0;
	BView drawview(rect, "draw_to_bmp", B_FOLLOW_NONE, B_WILL_DRAW);

	// crate Bitmap to Drag
	if (bmp->Lock())
	{
		bmp->AddChild(&drawview);

		// fill with transparent pixels
		drawview.SetDrawingMode(B_OP_COPY);
		drawview.SetLowColor(low_color);
		drawview.FillRect(rect, B_SOLID_LOW);

		//drawview.SetLowColor(make_rgb_color(0,0xff,0,0));
//drawview.FillRect(BRect(50,20,100,50), B_SOLID_LOW);

		// set font and color
		drawview.SetHighColor(high_color);
		drawview.SetFont(font);
		if (transparent_mode)
		{
			printf("makebitmap transparent mode B_OP_ALPHA");
			drawview.SetDrawingMode(B_OP_ALPHA);
			SetBlendingMode(B_PIXEL_ALPHA, B_ALPHA_OVERLAY);
		}
		else
		{
			printf("makebitmap not transparent mode B_OP_COPY");
			drawview.SetDrawingMode(B_OP_COPY);
		}

//drawview.FillRect(BRect(20,20,50,50), B_SOLID_HIGH);

		// compute where to draw string
		font_height fontheight;
		font->GetHeight(&fontheight);
		double baseline_y = rect.bottom -1.0 - fontheight.descent;
		edge_offset += BPoint(0 , baseline_y);

		// draw string
		drawview.DrawString(text, edge_offset);
		bmp->RemoveChild(&drawview);
		bmp->Unlock();
	}
	return bmp;
}

#if 0
void FontView::InitTranslators()
{
	//	Find the translator to use for us
	translator_id * all_translators = NULL;
	int32 count = 0;
	status_t err = BTranslatorRoster::Default()->GetAllTranslators(&all_translators, &count);
	if (err >= B_OK) {
//		this->all_translators_num = count;

		err = B_ERROR;
		//	look through all available translators
		for (int ix=0; ix<count; ix++) {
			const translation_format * in_formats;
			int32 fmt_count;
			if (B_OK <= BTranslatorRoster::Default()->GetInputFormats(all_translators[ix],
					&in_formats, &fmt_count)) {
				//	look for translators that accept BBitmaps as input
				for (int iy=0; iy<fmt_count; iy++) {
					if (in_formats[iy].type == B_TRANSLATOR_BITMAP) {
						goto do_this_format;
					}
				}
			}
			continue;
do_this_format:
			const translation_format * out_formats;
			if (B_OK <= BTranslatorRoster::Default()->GetOutputFormats(all_translators[ix],
					&out_formats, &fmt_count)) {
				//	look through the output formats
				for (int iy=0; iy<fmt_count; iy++) {
					printf("format = %s\n",out_formats[iy].MIME);
					string type(out_formats[iy].MIME);

					// add entry to map (key,value) = (mimetype, translaor)
					this->translators_map.insert(
						make_pair(type, make_pair(all_translators[ix], out_formats[iy].type)));

					printf("translator_id = %ld\n", all_translators[ix]);
					printf("translator_code = %ld\n", out_formats[iy].type);
					
/*					if (strcmp(out_formats[iy].MIME, "image/x-targa") == 0)
					{
						translator = all_translators[ix];
						translator_type_code = out_formats[iy].type;
						strcpy(translator_type_string, out_formats[iy].MIME);
						err = B_OK;
						break;
					}
*/					
					//	and take the first output format that isn't BBitmap
/*
					if (out_formats[iy].type != B_TRANSLATOR_BITMAP) {
						translator = all_translators[ix];
						translator_type_code = out_formats[iy].type;
						strcpy(translator_type_string, out_formats[iy].MIME);
						err = B_OK;
						break;
					}
*/
				}
			}
		}
	}
	delete[] all_translators;
	if (err < B_OK) {
	}
	MimetypeTranslatorMap::iterator it;
	for (it = translators_map.begin() ; it != translators_map.end() ; it++)
	{
		printf("translator: %s\n", (it->first).c_str());
	}
}
#endif

void FontView::SetDnDOutputFormat(const char *type)
{
	printf("FontView::SetDnDOutputFormat : %s\n", type);
	if (this->LockLooper())
	{
		dnd_output_format = type;
		this->UnlockLooper();
	}
}
