/*
	
	FontWorldConfig.cpp
	
	Copyright 1998 HAMANAKA Makoto , All Rights Reserved.
	
*/

#ifndef FONT_WORLD_CONFIG_H
#include "FontWorldConfig.h"
#endif

#ifndef FONT_DISPLAY_SETTING_H
#include "FontDisplaySetting.h"
#endif

#ifndef _STRING_H
#include "String.h"
#endif
#ifndef _MESSAGE_H
#include "Message.h"
#endif
#ifndef _FILE_H
#include "File.h"
#endif
#ifndef _PATH_H
#include "Path.h" 
#endif
#ifndef _FIND_DIRECTORY_H
#include "FindDirectory.h"
#endif

#include <stdio.h>

FontWorldConfig::FontWorldConfig()
	:configfile_path()
	,settings()
	,display_history_strings()
	,locker()
{
}

/*
bool FontWorldConfig::QuitRequested()
{
//	be_app->PostMessage(B_QUIT_REQUESTED);
//	return(true);
}
*/

status_t FontWorldConfig::Update()
{
	BMessage config_data;
	status_t status;

	this->Lock();

	/* add settings-data to config */

	int32 items_num = this->settings.CountItems();
	for (int i=0 ; i<items_num ; i++)
	{
		BMessage setting_data;
		FontDisplaySetting *setting;
		setting = (FontDisplaySetting *)this->settings.ItemAt(i);
		setting->SaveToMessage(&setting_data);
		config_data.AddMessage("settings",&setting_data);
	}
	
	/* todo: add history to config */
	
	BPath path;
	status = path.SetTo(configfile_path.String());
	if (status != B_OK)
	{// error
		printf("FontWorldConfig::Update() can't save config file(path)\n");
		this->Unlock();
		return B_ERROR;
	}
	
	BFile file;
	status = file.SetTo(path.Path(), B_WRITE_ONLY | B_CREATE_FILE); 
	if (status != B_OK)
	{// error
		printf("FontWorldConfig::Update() can't save config file(file)\n");
		this->Unlock();
		return B_ERROR;
	}
	
	status = config_data.Flatten(&file);
	if (status != B_OK)
	{// error
		printf("FontWorldConfig::Update() can't load config file(flatten)\n");
		this->Unlock();
		return B_ERROR;
	}
	
	this->Unlock();
	return B_OK;
}

status_t FontWorldConfig::Load()
{
	status_t status;
	BFile file;
	BMessage config_data;
	
	if (this->Lock())
	{

		BPath path;
		status = path.SetTo(configfile_path.String());
		if (status != B_OK)
		{// error
			printf("FontWorldConfig::Load() can't load config file(path)\n");
			this->Unlock();
			return B_ERROR;
		}
		
		status = file.SetTo(path.Path(), B_READ_ONLY); 
		if (status != B_OK)
		{// error
			printf("FontWorldConfig::Load() can't load config file\n");
			this->Unlock();
			return B_ERROR;
		}
	
		status = config_data.Unflatten(&file);
		if (status != B_OK)
		{
			printf("FontWorldConfig::Load config_data.Unflatten() error\n");
			this->Unlock();
			return B_ERROR;
		}
		
		int32 items_num = 0;
		type_code type;
		status = config_data.GetInfo("settings", &type, &items_num);
		if (status != B_OK)
		{
			printf("FontWorldConfig::Load config_data.GetInfo(\"settings\") error\n");
			items_num = 0;
		}
	
		for (int i=0 ; i<items_num ; i++)
		{
			BMessage setting_data;
			FontDisplaySetting *setting = new FontDisplaySetting();
			config_data.FindMessage("settings", i, &setting_data);
			setting->LoadFromMessage(&setting_data);
			settings.AddItem((void *)setting);
		}
		
		this->Unlock();
	}
	return B_OK;
	
}

status_t FontWorldConfig::AddSetting(FontDisplaySetting *_setting)
{
	bool status = false;
	if (this->Lock())
	{
		status = this->settings.AddItem((void *) new FontDisplaySetting(*_setting));
		this->Unlock();
	}
	return ((status) ? B_OK : B_ERROR);
}

status_t FontWorldConfig::RemoveSetting(FontDisplaySetting *_setting)
{
	bool status = false;
	if (this->Lock())
	{
		status = this->settings.RemoveItem((void *)_setting);
		this->Unlock();
	}
	return ((status) ? B_OK : B_ERROR);
}

status_t FontWorldConfig::GetSetting(BString _name, FontDisplaySetting **_found_setting)
{
	if (this->Lock())
	{
		bool found = false;
		int32 items_num = this->settings.CountItems();
		for (int i=0 ; i< items_num ; i++)
		{
			FontDisplaySetting *setting;
			BString setting_name;
			setting = (FontDisplaySetting *)this->settings.ItemAt(i);
			if (setting->GetSettingName() == _name)
			{
				found = true;
				*_found_setting = new FontDisplaySetting(*setting);
				break;
			}
		}
		if (!found)
		{
			this->Unlock();
			return B_NAME_NOT_FOUND;
		}	
		this->Unlock();
	}
	return B_OK;
}

status_t FontWorldConfig::CountSettings(int32 *settings_num)
{
	if (!this->locker.IsLocked())
	{
		printf("FontWorldConfig::CountSettings() needs locked access!\n");
		return B_ERROR;
	}
	*settings_num = this->settings.CountItems();
	return B_OK;
}

FontDisplaySetting * FontWorldConfig::SettingAt(int32 index)
{
	if (!this->locker.IsLocked())
	{
		printf("FontWorldConfig::SettingAt() needs locked access!\n");
		return 0;
	}
	return (FontDisplaySetting *)this->settings.ItemAt(index);
}

BString FontWorldConfig::SettingNameAt(int32 index)
{
	FontDisplaySetting *setting;
	if (!this->locker.IsLocked())
	{
		printf("FontWorldConfig::SettingNameAt() needs locked access!");
		return "";
	}
	setting = (FontDisplaySetting *)this->settings.ItemAt(index);
	return setting->GetSettingName();
}

/* --- setting ---
	bool enable_display_lines;
	bool enable_antialiasing;
	bool black_white_mode;
	float font_size;
	rgb_color fg_color;
	rgb_color bg_color;
*/

#ifndef RGB_COLOR_UTIL_H
#include "rgb_color_util.h"
#endif

void FontWorldConfig::test(void)
{
	FontWorldConfig config;
	FontDisplaySetting setting1;
	BPath path;
	status_t status;

	status = find_directory(B_COMMON_SETTINGS_DIRECTORY, &path); 
	if (status != B_OK)
	{
		printf("setting directory not found\n");
		return;
	}
	
	BString config_file_locaion;
	config_file_locaion.SetTo(path.Path());
	config_file_locaion += "/fontworld_test_setting";
	
	printf("config file location = %s\n", config_file_locaion.String());

	config.SetConfigFileLocation(config_file_locaion);

	setting1.SetSettingName("*setting1*");
	setting1.fg_color = make_rgb_color(60,50,40);
	setting1.bg_color = make_rgb_color(1,2,3);
	setting1.enable_display_lines = false;
	setting1.enable_antialiasing = true;
	setting1.black_white_mode = false;
	setting1.font_size = 100.0;

	status = config.AddSetting(&setting1);
	if (status != B_OK)
	{
		printf("config.AddSetting() error\n");
		return;
	}

	setting1.SetSettingName("setting_after");
	setting1.fg_color = make_rgb_color(20,30,40);
	setting1.bg_color = make_rgb_color(11,12,13);
	setting1.enable_display_lines = false;
	setting1.enable_antialiasing = true;
	setting1.black_white_mode = true;
	setting1.font_size = 300.0;
	
	status = config.Update();
	if (status != B_OK)
	{
		printf("config.Update() error\n");
		return;
	}

	status = config.AddSetting(&setting1);
	if (status != B_OK)
	{
		printf("config.AddSetting() error\n");
		return;
	}


	FontWorldConfig config_load;
	config_load.SetConfigFileLocation(config_file_locaion);
	status = config_load.Load();
	if (status != B_OK)
	{
		printf("config.Load() error\n");
		return;
	}
	
	FontDisplaySetting *tmp_setting;

	printf("------  config -------\n");
	if (config.Lock())
	{
		int32 settings_num = 0;
		status = config.CountSettings(&settings_num);
		if (status != B_OK)
		{
			printf("config.CountSettings() error\n");
			config.Unlock();
			return;
		}
		printf("::test() settings_num = %ld\n", settings_num);
		for (int i=0 ; i<settings_num ; i++)
		{
			printf(">>>> Setting(%d) name : <%s>\n", i, config.SettingNameAt(i).String());
			tmp_setting = config.SettingAt(i);
			printf("%s", tmp_setting->ToString().String());
		}

		config.Unlock();
	}
	printf("------  config_load -------\n");
	if (config_load.Lock())
	{
		int32 settings_num = 0;
		status = config_load.CountSettings(&settings_num);
		if (status != B_OK)
		{
			printf("config.CountSettings() error\n");
			config_load.Unlock();
			return;
		}
		printf("::test() settings_num = %ld\n", settings_num);
		for (int i=0 ; i<settings_num ; i++)
		{
			printf(">>>> Setting(%d) name : <%s>\n", i, config_load.SettingNameAt(i).String());
			tmp_setting = config_load.SettingAt(i);
			printf("%s", tmp_setting->ToString().String());
		}

		config_load.Unlock();
	}
	
	printf("FontWorldConfig::test() finished\n\n");
	fflush(stdout);
}

#if 0
//-------------   save config example
	FontWorldConfig config;
	FontDisplaySetting setting1;
	BPath path;
	status_t status;

	status = find_directory(B_COMMON_SETTINGS_DIRECTORY, &path); 
	if (status != B_OK)
	{
		printf("setting directory not found\n");
		return;
	}
	
	BString config_file_locaion;
	config_file_locaion.SetTo(path.Path());
	config_file_locaion += "/fontworld_test_setting";
	
	printf("config file location = %s\n", config_file_locaion.String());

	config.SetConfigFileLocation(config_file_locaion);

	setting1.SetSettingName("*setting1*");
	setting1.fg_color = make_rgb_color(60,50,40);
	setting1.bg_color = make_rgb_color(1,2,3);
	setting1.enable_display_lines = false;
	setting1.enable_antialiasing = true;
	setting1.black_white_mode = false;
	setting1.font_size = 100.0;

	status = config.AddSetting(&setting1);
	if (status != B_OK)
	{
		printf("config.AddSetting() error\n");
		return;
	}

	status = config.Update();
	if (status != B_OK)
	{
		printf("config.Update() error\n");
		return;
	}

//-------------   load config example

	FontWorldConfig config;
	FontDisplaySetting setting1;
	BPath path;
	status_t status;

	status = find_directory(B_COMMON_SETTINGS_DIRECTORY, &path); 
	if (status != B_OK)
	{
		printf("setting directory not found\n");
		return;
	}
	
	BString config_file_locaion;
	config_file_locaion.SetTo(path.Path());
	config_file_locaion += "/fontworld_test_setting";
	
	printf("config file location = %s\n", config_file_locaion.String());

	config.SetConfigFileLocation(config_file_locaion);

	status = config.Load();
	if (status != B_OK)
	{
		printf("config.Load() error\n");
		return;
	}

	/* unfold settings */
	if (config.Lock())
	{
		int32 settings_num = 0;
		status = config.CountSettings(&settings_num);
		if (status != B_OK)
		{
			printf("config.CountSettings() error\n");
			config.Unlock();
			return;
		}
		printf("::test() settings_num = %ld\n", settings_num);
		for (int i=0 ; i<settings_num ; i++)
		{
			printf(">>>> Setting(%d) name : <%s>\n", i, config.SettingNameAt(i).String());
			tmp_setting = config.SettingAt(i);
			printf("%s", tmp_setting->ToString().String());
		}

		config.Unlock();
	}

#endif
